"use strict";

// Import dependencies
import("/js/core.js").then(() =>
	import("/js/damper.js").then(() => {
		const dependencies = setInterval(() => {
			if (
				typeof core === "function" &&
				typeof core.damper === "function"
			) {
				clearInterval(dependencies);
				clearTimeout(timeout);
				initialization();
			}
		}, 10);
		const timeout = setTimeout(() => {
			clearInterval(dependencies);
			initialization();
		}, 5000);

		function initialization() {
			if (typeof core.cart === "undefined") {
				// Not initialized

				/**
				 * @name Cart
				 *
				 * @description
				 * Implements actions with cart
				 *
				 * @memberof core
				 *
				 * @license http://www.wtfpl.net/ Do What The Fuck You Want To Public License
				 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
				 */
				core.cart = class cart {
					/**
					 * Toggle
					 *
					 * Toggle the product in the cart (interface)
					 *
					 * @param {HTMLButtonElement|HTMLInputElement} element Handler elememnt of the product
					 * @param {HTMLElement} product The product
					 * @param {bool} remove Remove the product element if json.amount === 0?
					 * @param {bool} force Ignore the damper? (false)
					 *
					 * @return {bool} True if an error occurs to continue the event execution
					 */
					static toggle(element, product, remove = false, force = false) {
						// Blocking the element
						element.setAttribute("disabled", "true");

						// Execute under damper
						this.toggle_damper(
							element,
							product,
							"toggle",
							undefined,
							remove,
							force,
						);

						// Exit (success)
						return false;
					}

					/**
					 * Toggle
					 *
					 * Toggle the product in the cart (damper)
					 *
					 * @param {HTMLButtonElement|HTMLInputElement} element Handler elememnt of the product
					 * @param {HTMLElement} product The product
					 * @param {bool} remove Remove the product element if json.amount === 0?
					 * @param {bool} force Ignore the damper? (false)
					 *
					 * @return {Promise}
					 */
					static toggle_damper = core.damper(
						(...variables) => this.product(...variables).then(this.summary),
						300,
						6,
					);

					/**
					 * Write
					 *
					 * Write the product in the cart (interface)
					 *
					 * @param {HTMLButtonElement|HTMLInputElement} element Handler elememnt of the product
					 * @param {HTMLElement} product The product
					 * @param {number} amount Amount of writings
					 * @param {bool} remove Remove the product element if json.amount === 0?
					 * @param {bool} force Ignore the damper? (false)
					 *
					 * @return {bool} True if an error occurs to continue the event execution
					 */
					static write(
						element,
						product,
						amount = 1,
						remove = false,
						force = false,
					) {
						// Blocking the element
						element.setAttribute("disabled", "true");

						// Execute under damper
						this.write_damper(element, product, "write", amount, remove, force);

						// Exit (success)
						return false;
					}

					/**
					 * Write
					 *
					 * Write the product in the cart (damper)
					 *
					 * @param {HTMLButtonElement|HTMLInputElement} element Handler elememnt of the product
					 * @param {HTMLElement} product The product
					 * @param {number} amount Amount of writings
					 * @param {bool} remove Remove the product element if json.amount === 0?
					 * @param {bool} force Ignore the damper? (false)
					 *
					 * @return {Promise}
					 */
					static write_damper = core.damper(
						(...variables) => this.product(...variables).then(this.summary),
						300,
						6,
					);

					/**
					 * Delete
					 *
					 * Delete the product from the cart (interface)
					 *
					 * @param {HTMLButtonElement|HTMLInputElement} element Handler elememnt of the product
					 * @param {HTMLElement} product The product
					 * @param {number} amount Amount of deletings
					 * @param {bool} remove Remove the product element if json.amount === 0?
					 * @param {bool} force Ignore the damper? (false)
					 *
					 * @return {bool} True if an error occurs to continue the event execution
					 */
					static delete(
						element,
						product,
						amount = 1,
						remove = false,
						force = false,
					) {
						// Blocking the element
						element.setAttribute("disabled", "true");

						// Execute under damper
						this.delete_damper(
							element,
							product,
							"delete",
							amount,
							remove,
							force,
						);

						// Exit (success)
						return false;
					}

					/**
					 * Delete
					 *
					 * Delete the product from the cart (damper)
					 *
					 * @param {HTMLButtonElement|HTMLInputElement} element Handler elememnt of the product
					 * @param {number} amount Amount of deletings
					 * @param {bool} remove Remove the product element if json.amount === 0?
					 * @param {bool} force Ignore the damper? (false)
					 *
					 * @return {Promise}
					 */
					static delete_damper = core.damper(
						(...variables) => this.product(...variables).then(this.summary),
						300,
						6,
					);

					/**
					 * Set
					 *
					 * Set amount of the product in the cart (interface)
					 *
					 * @param {HTMLButtonElement|HTMLInputElement} element Handler elememnt of the product
					 * @param {HTMLElement} product The product
					 * @param {number} amount Amount of the product in the cart to be setted
					 * @param {bool} remove Remove the product element if json.amount === 0?
					 * @param {bool} force Ignore the damper? (false)
					 *
					 * @return {bool} True if an error occurs to continue the event execution
					 */
					static set(
						element,
						product,
						amount = 1,
						remove = false,
						force = false,
					) {
						// Blocking the element
						element.setAttribute("disabled", "true");

						// Execute under damper
						this.set_damper(element, product, "set", amount, remove, force);

						// Exit (success)
						return false;
					}

					/**
					 * Set
					 *
					 * Set the product in the cart (damper)
					 *
					 * @param {HTMLButtonElement|HTMLInputElement} element Handler elememnt of the product
					 * @param {HTMLElement} product The product
					 * @param {number} amount Amount of the product in the cart to be setted
					 * @param {bool} remove Remove the product element if json.amount === 0?
					 * @param {bool} force Ignore the damper? (false)
					 *
					 * @return {Promise}
					 */
					static set_damper = core.damper(
						(...parameters) => this.product(...parameters).then(this.summary),
						300,
						6,
					);

					/**
					 * The product
					 *
					 * Handle the product in the cart (system)
					 *
					 * @param {HTMLButtonElement|HTMLInputElement} element Handler element of the product
					 * @param {HTMLElement} product The product element
					 * @param {string} type Type of action with the product
					 * @param {number} amount Amount of product to handle
					 * @param {bool} remove Remove the product element if json.amount === 0?
					 *
					 * @return {Promise|null}
					 */
					static async product(
						element,
						product,
						type,
						amount = null,
						remove = false,
						resolve = () => {},
						reject = () => {},
					) {
						if (
							(element instanceof HTMLButtonElement ||
								element instanceof HTMLInputElement) &&
							product instanceof HTMLElement
						) {
							// Validated

							// Initializing the buffer of request body
							let request = "";

							// Initializing of identifier of the product
							const identifier = +product.getAttribute(
								"data-product-identifier",
							);

							if (typeof identifier === "number") {
								// Validated identifier

								// Writing to the buffer of request body
								request += "&identifier=" + identifier;

								if (
									type === "toggle" ||
									type === "write" ||
									type === "delete" ||
									type === "set"
								) {
									// Validated type

									// Writing to the buffer of request body
									request += "&type=" + type;

									if (
										(type === "toggle" &&
											typeof amount === "undefined") ||
										(type === "set" &&
												amount === 0 ||
											amount === 100) ||
										typeof amount === "number" &&
											amount > 0 &&
											amount < 100
									) {
										// Validated amount

										if (type !== "toggle") {
											// Not a toggle request

											// Writing to the buffer of request body
											request += "&amount=" + amount;
										}

										// Request
										return await core.request(
											"/cart/product",
											request,
										)
											.then((json) => {
												if (
													json.errors !== null &&
													typeof json.errors === "object" &&
													json.errors.length > 0
												) {
													// Fail (received errors)
												} else {
													// Success (not received errors)

													if (remove && json.amount === 0) {
														// Requested deleting of the product element when there is no the product in the cart

														// Deleting the product element
														product.remove();
													} else {
														// Not requested deleting the product element when there is no the product in the cart

														// Unblocking the element
														element.removeAttribute("disabled");

														// Writing offset of hue-rotate to indicate that the product is in the cart
														product.style.setProperty(
															"--hue-rotate-offset",
															json.amount + "0deg",
														);

														// Writing attribute with amount of the product in the cart
														product.setAttribute(
															"data-product-amount",
															json.amount,
														);

														// Initializing the amount <span> element
														const amounts = product.querySelectorAll(
															'[data-product-parameter="amount"]',
														);

														for (const amount of amounts) {
															// Iterating over an amount elements

															if (amount instanceof HTMLInputElement) {
																// The <input> element

																// Writing amount of the product in the cart
																amount.value = json.amount;
															} else {
																// Not the <input> element

																// Writing amount of the product in the cart
																amount.innerText = json.amount;
															}
														}

														// Exit (success)
														resolve();
													}
												}
											});
									}
								}
							}
						}

						// Exit (fail)
						reject();
					}

					/**
					 * Summary
					 *
					 * Initialize summary of products the cart (system)
					 *
					 * @return {void}
					 */
					static async summary() {
						// Request
						return await core.request("/cart/summary")
							.then((json) => {
								if (
									json.errors !== null &&
									typeof json.errors === "object" &&
									json.errors.length > 0
								) {
									// Fail (received errors)
								} else {
									// Success (not received errors)

									// Initializing the summary amount <span> element
									const amount = document.getElementById("amount");

									// Initializing the summary cost <span> element
									const cost = document.getElementById("cost");

									if (amount instanceof HTMLElement) {
										// Initialized the summary amount element

										// Writing summmary amount into the summary amount element
										amount.innerText = json.amount;
									}

									if (cost instanceof HTMLElement) {
										// Initialized the summary cost element

										// Writing summmary cost into the summary cost element
										cost.innerText = json.cost;
									}
								}
							});
					}
				};
			}
		}
	})
);
